import { EventFormatter } from '../util';
import { Channel } from './channel';
//import * as SocketCluster from "socketcluster";
//import {SCChannel} from "sc-channel";
import {SCClientSocket} from "socketcluster-client";
import {SCChannel} from "sc-channel";

/**
 * This class represents a Socket Cluster channel.
 */
export class SocketClusterChannel extends Channel {
    /**
     * The SocketCluster client instance.
     */
    socket: SCClientSocket;

    /**
     * The name of the channel.
     */
    name: any;

    /**
     * Channel options.
     */
    options: any;

    /**
     * The event formatter.
     */
    eventFormatter: EventFormatter;

    /**
     * The event callbacks applied to the channel.
     */
    events: any = {};

    /**
     * User supplied callbacks for events on this channel.
     */
    private listeners: any = {};

    channelObject: SCChannel;

    connectedOnce: boolean = false;

    /**
     * Create a new class instance.
     */
    constructor(socket: SCClientSocket, name: string, options: any) {
        super();

        this.name = name;
        this.socket = socket;
        this.options = options;
        this.eventFormatter = new EventFormatter(this.options.namespace);

        this.subscribe();
    }

    /**
     * Subscribe to a socket-cluster channel.
     */
    subscribe(): void {
        let authParams = this.options.auth || {};
        authParams.url = this.getFullAuthEndpoint();
        this.channelObject = this.socket.subscribe(this.name, {
            data: {
                channel: this.name,
                auth: authParams
            }
        });
        this.channelObject.watch(data => {
            if(typeof this.events[data.event] !== typeof undefined){
                this.events[data.event].forEach(callback => {
                    callback(data.data);
                });
            }
        });
    }

    getFullAuthEndpoint(){
        let protocol = window.location.protocol;
        if(this.options.auth['secure'] || this.options.secure){
            protocol = "https:"
        }
        var hostname = this.options.auth['hostname'] || (this.options.socketcluster['hostname'] || window.location.hostname);
        let host = protocol+'//'+hostname;
        return host+this.options.authEndpoint;
    }

    /**
     * Unsubscribe from channel and ubind event callbacks.
     */
    unsubscribe(): void {
        this.unbind();
        this.channelObject.unwatch();
        this.socket.unsubscribe(this.name);
        this.socket.destroyChannel(this.name);
    }

    /**
     * Listen for an event on the channel instance.
     */
    listen(event: string, callback: Function): SocketClusterChannel {
        this.on(this.eventFormatter.format(event), callback);

        return this;
    }

    /**
     * Stop listening for an event on the channel instance.
     */
    stopListening(event: string, callback?: Function): SocketClusterChannel {
        this.unbindEvent(this.eventFormatter.format(event), callback);

        return this;
    }

    /**
     * Register a callback to be called anytime a subscription succeeds.
     */
    subscribed(callback: Function): SocketClusterChannel {
        const listener = (socket) => {
            /*if(!this.connectedOnce){
                this.connectedOnce = true;
                return;
            }*/
            callback(socket);
        };

        this.on('connect', listener);
        return this;
    }

    /**
     * Register a callback to be called anytime an error occurs.
     */
    error(callback: Function): SocketClusterChannel {
        return this;
    }

    /**
     * Bind the channel's socket to an event and store the callback.
     */
    on(event: string, callback: Function): SocketClusterChannel {
        this.listeners[event] = this.listeners[event] || [];

        if (!this.events[event]) {
            this.events[event] = (channel, data) => {
                if (this.name === channel && this.listeners[event]) {
                    this.listeners[event].forEach((cb) => cb(data));
                }
            };

            this.channelObject.on(event, this.events[event]);
        }

        this.listeners[event].push(callback);

        return this;
    }

    /**
     * Unbind the channel's socket from all stored event callbacks.
     */
    unbind(): void {
        Object.keys(this.events).forEach((event) => {
            this.unbindEvent(event);
        });
    }

    /**
     * Unbind the listeners for the given event.
     */
    protected unbindEvent(event: string, callback?: Function): void {
        this.listeners[event] = this.listeners[event] || [];

        if (callback) {
            this.listeners[event] = this.listeners[event].filter((cb) => cb !== callback);
        }

        if (!callback || this.listeners[event].length === 0) {
            if (this.events[event]) {
                this.events[event].forEach(callback => {
                    this.channelObject.off(event, callback);
                });

                delete this.events[event];
            }

            delete this.listeners[event];
        }
    }
}
